package arzumify.coretechs;

import arzumify.coretechs.energy.*;
import eu.pb4.polymer.core.api.item.PolymerBlockItem;
import net.fabricmc.fabric.api.itemgroup.v1.ItemGroupEvents;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemGroups;
import net.minecraft.item.Items;
import net.minecraft.registry.Registries;
import net.minecraft.registry.Registry;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.sound.BlockSoundGroup;
import net.minecraft.util.Identifier;

public class CoreBlocks {
    public static <T extends Block> T register(T block, RegistryKey<Block> blockKey, boolean shouldRegisterItem, Identifier modelPath) {
        if (shouldRegisterItem) {
            RegistryKey<Item> itemKey = RegistryKey.of(RegistryKeys.ITEM, blockKey.getValue());
            CoreTechs.LOGGER.info("Registering block item: {}", itemKey.getValue().getPath());
            PolymerBlockItem blockItem = new TexturedBlockItem(block, new Item.Settings(), Items.TRIAL_KEY, modelPath);
            Registry.register(Registries.ITEM, itemKey, blockItem);
        }

        return Registry.register(Registries.BLOCK, blockKey, block);
    }

    public static void initialize() {
        ItemGroupEvents.modifyEntriesEvent(ItemGroups.REDSTONE).register((itemGroup) -> itemGroup.add(COPPER_WIRE.asItem()));
        ItemGroupEvents.modifyEntriesEvent(ItemGroups.REDSTONE).register((itemGroup) -> itemGroup.add(CREATIVE_GENERATOR.asItem()));
        ItemGroupEvents.modifyEntriesEvent(ItemGroups.REDSTONE).register((itemGroup) -> itemGroup.add(VOLTMETER.asItem()));
        ItemGroupEvents.modifyEntriesEvent(ItemGroups.REDSTONE).register((itemGroup) -> itemGroup.add(MAGNET.asItem()));
        ItemGroupEvents.modifyEntriesEvent(ItemGroups.REDSTONE).register((itemGroup) -> itemGroup.add(SUMMONER.asItem()));
    }

    public static final RegistryKey<Block> COPPER_WIRE_KEY = RegistryKey.of(
            RegistryKeys.BLOCK,
            Identifier.of(CoreTechs.MOD_ID, "copper_wire")
    );
    public static final CopperWire COPPER_WIRE = register(
            new CopperWire(
                    AbstractBlock.Settings.create().
                            sounds(BlockSoundGroup.WOOL)
            ),
            COPPER_WIRE_KEY,
            true,
            Identifier.of(CoreTechs.MOD_ID, "block/copper_wire")
    );

    public static final RegistryKey<Block> CREATIVE_GENERATOR_KEY = RegistryKey.of(
            RegistryKeys.BLOCK,
            Identifier.of(CoreTechs.MOD_ID, "creative_generator")
    );
    public static final CreativeGenerator CREATIVE_GENERATOR = register(
            new CreativeGenerator(
                    AbstractBlock.Settings.create().
                            sounds(BlockSoundGroup.METAL)
            ),
            CREATIVE_GENERATOR_KEY,
            true,
            Identifier.of(CoreTechs.MOD_ID, "block/creative_generator")
    );

    public static final RegistryKey<Block> VOLTMETER_KEY = RegistryKey.of(
            RegistryKeys.BLOCK,
            Identifier.of(CoreTechs.MOD_ID, "voltmeter")
    );
    public static final Voltmeter VOLTMETER = register(
            new Voltmeter(
                    AbstractBlock.Settings.create().
                            sounds(BlockSoundGroup.METAL)
            ),
            VOLTMETER_KEY,
            true,
            Identifier.of(CoreTechs.MOD_ID, "block/voltmeter")
    );

    public static final RegistryKey<Block> MAGNET_KEY = RegistryKey.of(
            RegistryKeys.BLOCK,
            Identifier.of(CoreTechs.MOD_ID, "magnet")
    );
    public static final Magnet MAGNET = register(
            new Magnet(
                    AbstractBlock.Settings.create().
                            sounds(BlockSoundGroup.METAL)
            ),
            MAGNET_KEY,
            true,
            Identifier.of(CoreTechs.MOD_ID, "block/magnet")
    );

    public static final RegistryKey<Block> SUMMONER_KEY = RegistryKey.of(
            RegistryKeys.BLOCK,
            Identifier.of(CoreTechs.MOD_ID, "summoner")
    );
    public static final Summoner SUMMONER = register(
            new Summoner(
                    AbstractBlock.Settings.create().
                            sounds(BlockSoundGroup.METAL)
            ),
            SUMMONER_KEY,
            true,
            Identifier.of(CoreTechs.MOD_ID, "block/summoner")
    );
}