package arzumify.coretechs.energy;

import arzumify.coretechs.BlockEntityTypes;
import arzumify.polyenergy.api.EnergyProvider;
import arzumify.polyenergy.api.EnergyReceiver;
import arzumify.located.math.Vec3i;
import arzumify.polyenergy.impl.MatchMaker;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerBlockEntityEvents;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerTickEvents;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class CopperWireEntity extends BlockEntity implements EnergyProvider, EnergyReceiver, ServerBlockEntityEvents.Unload, ServerBlockEntityEvents.Load, ServerTickEvents.EndWorldTick {
    public CopperWireEntity(BlockPos pos, BlockState state) {
        super(BlockEntityTypes.COPPER_WIRE, pos, state);
        ServerBlockEntityEvents.BLOCK_ENTITY_UNLOAD.register(this);
        ServerBlockEntityEvents.BLOCK_ENTITY_LOAD.register(this);
        ServerTickEvents.END_WORLD_TICK.register(this);
    }

    private final Set<EnergyProvider> providers = ConcurrentHashMap.newKeySet();
    private final Set<EnergyReceiver> receivers = ConcurrentHashMap.newKeySet();
    private final Set<EnergyReceiver> blackListed = ConcurrentHashMap.newKeySet();
    private boolean active = true;
    private long energy = 0;

    public static void tick(World ignoredWorld, BlockPos ignoredPos, BlockState ignoredState, CopperWireEntity wire) {
        wire.energy = Common.extract(1, 16, wire.energy, wire.providers, wire);
        
        if (!wire.receivers.isEmpty() && wire.energy > 0 && wire.active) {
            wire.energy = Common.extract(1, 16, wire.energy, wire.providers, wire);
        } else {
            System.out.println("Fails conditions: " + wire.receivers.isEmpty() + " " + wire.energy + " " + wire.active);
        }
    }

    @Override
    public long extract(long amount, EnergyReceiver receiver) {
        System.out.println("Giving " + amount + " to " + receiver);
        if (!blackListed.contains(receiver)) {
            System.out.println("Not blacklisted");
            var extracted = Math.min(amount, Math.min(1, energy));
            System.out.println("Extracted " + extracted);
            energy -= extracted;
            System.out.println("Energy is now " + energy);
            blackListed.add(receiver);
            return extracted;
        } else {
            return 0;
        }
    }

    @Override
    public void ready(EnergyReceiver receiver) {
        receivers.add(receiver);
        receiver.ready(this);
    }

    @Override
    public void ready(EnergyProvider provider) {
        providers.add(provider);
    }

    @Override
    public void unready(EnergyReceiver receiver) {
        receivers.remove(receiver);
    }

    @Override
    public void unready(EnergyProvider provider) {
        providers.remove(provider);
    }

    @Override
    public void onLoad(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Add(this);
        this.active = true;
        reload();
    }



    @Override
    public void onUnload(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Remove(this);
        this.active = false;
        for (EnergyReceiver receiver : receivers) {
            receiver.unready(this);
        }
        for (EnergyProvider provider : providers) {
            provider.unready(this);
        }
        providers.clear();
        receivers.clear();
    }

    @Override
    public void onEndTick(ServerWorld world) {
        blackListed.clear();
    }

    @Override
    public Collection<Vec3i> points() {
        return List.of(new Vec3i(pos.getX(), pos.getY(), pos.getZ()));
    }

    @Override
    public String dimension() {
        return Objects.requireNonNull(this.getWorld()).getRegistryKey().getValue().toString();
    }

    public void reload() {
        var found = MatchMaker.matchMaker.Search(this);
        if (found != null) {
            for (var foundEnergyBlocks : found) {
                if (foundEnergyBlocks instanceof EnergyProvider && !providers.contains(foundEnergyBlocks)) {
                    ((EnergyProvider) foundEnergyBlocks).ready(this);
                }
                if (foundEnergyBlocks instanceof EnergyReceiver && !receivers.contains(foundEnergyBlocks)) {
                    receivers.add((EnergyReceiver) foundEnergyBlocks);
                    ((EnergyReceiver) foundEnergyBlocks).ready(this);
                }
            }
        }
    }
}
