package arzumify.coretechs.energy;

import arzumify.coretechs.BlockEntityTypes;
import arzumify.polyenergy.api.EnergyProvider;
import arzumify.polyenergy.api.EnergyReceiver;
import arzumify.polyenergy.impl.MatchMaker;
import arzumify.located.math.Vec3i;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerBlockEntityEvents;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.BlockPos;

import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class CreativeGeneratorEntity extends BlockEntity implements EnergyProvider, ServerBlockEntityEvents.Unload, ServerBlockEntityEvents.Load {
    public CreativeGeneratorEntity(BlockPos pos, BlockState state) {
        // add event listener
        super(BlockEntityTypes.CREATIVE_GENERATOR, pos, state);
        ServerBlockEntityEvents.BLOCK_ENTITY_UNLOAD.register(this);
        ServerBlockEntityEvents.BLOCK_ENTITY_LOAD.register(this);
    }

    public final Set<EnergyReceiver> receivers = ConcurrentHashMap.newKeySet();

    @Override
    public void ready(EnergyReceiver energyReceiver) {
        System.out.println("Ready " + energyReceiver);
        receivers.add(energyReceiver);
        energyReceiver.ready(this);
    }

    @Override
    public void unready(EnergyReceiver energyReceiver) {
        System.out.println("Unready " + energyReceiver);
        receivers.remove(energyReceiver);
    }

    @Override
    public void onLoad(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Add(this);
    }

    @Override
    public void onUnload(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Remove(this);
        for (EnergyReceiver receiver : receivers) {
            receiver.unready(this);
        }
    }

    @Override
    public long extract(long l, EnergyReceiver energyReceiver) {
        return Long.MAX_VALUE;
    }

    @Override
    public Collection<Vec3i> points() {
        return List.of(new Vec3i(pos.getX(), pos.getY(), pos.getZ()));
    }

    @Override
    public String dimension() {
        return Objects.requireNonNull(this.getWorld()).getRegistryKey().getValue().toString();
    }
}
