package arzumify.coretechs.energy;

import arzumify.coretechs.BlockEntityTypes;
import arzumify.coretechs.CoreTechs;
import arzumify.polyenergy.api.EnergyProvider;
import arzumify.polyenergy.api.EnergyReceiver;
import arzumify.polyenergy.impl.MatchMaker;
import arzumify.located.math.Vec3i;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerBlockEntityEvents;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.entity.ExperienceOrbEntity;
import net.minecraft.entity.EyeOfEnderEntity;
import net.minecraft.entity.ItemEntity;
import net.minecraft.entity.projectile.thrown.EnderPearlEntity;
import net.minecraft.entity.projectile.thrown.ExperienceBottleEntity;
import net.minecraft.entity.projectile.thrown.PotionEntity;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Box;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;

import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

public class MagnetEntity extends BlockEntity implements EnergyReceiver, ServerBlockEntityEvents.Unload, ServerBlockEntityEvents.Load {
    private final Set<EnergyProvider> providers = ConcurrentHashMap.newKeySet();
    private long energy = 0;
    private boolean active = true;
    private static final double multiplier = 0.05;

    public MagnetEntity(BlockPos pos, BlockState state) {
        super(BlockEntityTypes.MAGNET, pos, state);
        ServerBlockEntityEvents.BLOCK_ENTITY_UNLOAD.register(this);
        ServerBlockEntityEvents.BLOCK_ENTITY_LOAD.register(this);
    }

    @Override
    public void ready(EnergyProvider energyProvider) {
        System.out.println("Ready " + energyProvider);
        providers.add(energyProvider);
    }

    @Override
    public void unready(EnergyProvider energyProvider) {
        System.out.println("Unready " + energyProvider);
        providers.remove(energyProvider);
    }

    public static void tick(World world, BlockPos pos, BlockState ignoredState, MagnetEntity magnet) {
        System.out.println("Trying to extract energy");
        magnet.energy = Common.extract(1, 16, magnet.energy, magnet.providers, magnet);

        System.out.println("Tick energy: " + magnet.energy);
        if (magnet.energy >= 1 && magnet.active) {
            System.out.println("Magnet activated");
            magnet.energy -= 1;
            var entities = world.getOtherEntities(null, new Box(pos).expand(10));
            for (var entity : entities) {
                var direction = Vec3d.of(pos).subtract(entity.getPos()).normalize();
                if (entity instanceof ServerPlayerEntity) {
                    CoreTechs.sendVelocityDelta((ServerPlayerEntity) entity, direction.multiply(multiplier));
                } else if (entity instanceof EnderPearlEntity || entity instanceof EyeOfEnderEntity || entity instanceof ItemEntity || entity instanceof ExperienceOrbEntity || entity instanceof ExperienceBottleEntity || entity instanceof PotionEntity) {
                    entity.setNoGravity(true);
                    entity.setVelocity(direction.multiply(multiplier * 5));
                } else {
                    entity.setVelocity(direction.multiply(multiplier));
                }
            }
        }
    }

    @Override
    public void onLoad(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Add(this);
        active = true;
        reload();
    }

    @Override
    public void onUnload(BlockEntity blockEntity, ServerWorld serverWorld) {
        MatchMaker.matchMaker.Remove(this);
        active = false;
        for (EnergyProvider provider : providers) {
            provider.unready(this);
        }
    }

    @Override
    public Collection<Vec3i> points() {
        return List.of(new Vec3i(pos.getX(), pos.getY(), pos.getZ()));
    }

    @Override
    public String dimension() {
        return Objects.requireNonNull(this.getWorld()).getRegistryKey().getValue().toString();
    }

    public void reload() {
        var found = MatchMaker.matchMaker.Search(this);
        if (found != null) {
            for (var foundEnergyBlocks : found) {
                if (foundEnergyBlocks instanceof EnergyProvider && !providers.contains(foundEnergyBlocks)) {
                    ((EnergyProvider) foundEnergyBlocks).ready(this);
                }
            }
        }
    }
}
